import * as AST6 from '@eslint-react/ast';
import { flip, dual, constFalse, identity, _ } from '@eslint-react/eff';
import * as JSX3 from '@eslint-react/jsx';
import { DEFAULT_ESLINT_REACT_SETTINGS as DEFAULT_ESLINT_REACT_SETTINGS$1, unsafeDecodeSettings, getId } from '@eslint-react/shared';
import { AST_NODE_TYPES } from '@typescript-eslint/types';
import * as VAR from '@eslint-react/var';
import { match, P } from 'ts-pattern';
import '@eslint-react/kit';
import birecord from 'birecord';

// src/component/component-collector.ts

// src/constants.ts
var DISPLAY_NAME_ASSIGNMENT_SELECTOR = [
  "AssignmentExpression",
  "[type]",
  "[operator='=']",
  "[left.type='MemberExpression']",
  "[left.property.name='displayName']"
].join("");

// src/hook/hook-name.ts
var RE_HOOK_NAME = /^use[A-Z\d]/u;
function isReactHookName(name) {
  return name === "use" || RE_HOOK_NAME.test(name);
}
function isReactHookNameLoose(name) {
  return name.startsWith("use");
}

// ../../node_modules/.pnpm/valibot@1.0.0_typescript@5.8.2/node_modules/valibot/dist/index.js
var store;
// @__NO_SIDE_EFFECTS__
function getGlobalConfig(config2) {
  return {
    lang: config2?.lang ?? store?.lang,
    message: config2?.message,
    abortEarly: config2?.abortEarly ?? store?.abortEarly,
    abortPipeEarly: config2?.abortPipeEarly ?? store?.abortPipeEarly
  };
}
var store2;
// @__NO_SIDE_EFFECTS__
function getGlobalMessage(lang) {
  return store2?.get(lang);
}
var store3;
// @__NO_SIDE_EFFECTS__
function getSchemaMessage(lang) {
  return store3?.get(lang);
}
var store4;
// @__NO_SIDE_EFFECTS__
function getSpecificMessage(reference, lang) {
  return store4?.get(reference)?.get(lang);
}
// @__NO_SIDE_EFFECTS__
function _stringify(input) {
  const type = typeof input;
  if (type === "string") {
    return `"${input}"`;
  }
  if (type === "number" || type === "bigint" || type === "boolean") {
    return `${input}`;
  }
  if (type === "object" || type === "function") {
    return (input && Object.getPrototypeOf(input)?.constructor?.name) ?? "null";
  }
  return type;
}
function _addIssue(context, label, dataset, config2, other) {
  const input = other && "input" in other ? other.input : dataset.value;
  const expected = other?.expected ?? context.expects ?? null;
  const received = other?.received ?? /* @__PURE__ */ _stringify(input);
  const issue = {
    kind: context.kind,
    type: context.type,
    input,
    expected,
    received,
    message: `Invalid ${label}: ${expected ? `Expected ${expected} but r` : "R"}eceived ${received}`,
    requirement: context.requirement,
    path: other?.path,
    issues: other?.issues,
    lang: config2.lang,
    abortEarly: config2.abortEarly,
    abortPipeEarly: config2.abortPipeEarly
  };
  const isSchema = context.kind === "schema";
  const message = other?.message ?? context.message ?? /* @__PURE__ */ getSpecificMessage(context.reference, issue.lang) ?? (isSchema ? /* @__PURE__ */ getSchemaMessage(issue.lang) : null) ?? config2.message ?? /* @__PURE__ */ getGlobalMessage(issue.lang);
  if (message !== void 0) {
    issue.message = typeof message === "function" ? (
      // @ts-expect-error
      message(issue)
    ) : message;
  }
  if (isSchema) {
    dataset.typed = false;
  }
  if (dataset.issues) {
    dataset.issues.push(issue);
  } else {
    dataset.issues = [issue];
  }
}
// @__NO_SIDE_EFFECTS__
function _getStandardProps(context) {
  return {
    version: 1,
    vendor: "valibot",
    validate(value2) {
      return context["~run"]({ value: value2 }, /* @__PURE__ */ getGlobalConfig());
    }
  };
}
var ValiError = class extends Error {
  /**
   * Creates a Valibot error with useful information.
   *
   * @param issues The error issues.
   */
  constructor(issues) {
    super(issues[0].message);
    this.name = "ValiError";
    this.issues = issues;
  }
};
// @__NO_SIDE_EFFECTS__
function getFallback(schema, dataset, config2) {
  return typeof schema.fallback === "function" ? (
    // @ts-expect-error
    schema.fallback(dataset, config2)
  ) : (
    // @ts-expect-error
    schema.fallback
  );
}
// @__NO_SIDE_EFFECTS__
function getDefault(schema, dataset, config2) {
  return typeof schema.default === "function" ? (
    // @ts-expect-error
    schema.default(dataset, config2)
  ) : (
    // @ts-expect-error
    schema.default
  );
}
// @__NO_SIDE_EFFECTS__
function array(item, message) {
  return {
    kind: "schema",
    type: "array",
    reference: array,
    expects: "Array",
    async: false,
    item,
    message,
    get "~standard"() {
      return /* @__PURE__ */ _getStandardProps(this);
    },
    "~run"(dataset, config2) {
      const input = dataset.value;
      if (Array.isArray(input)) {
        dataset.typed = true;
        dataset.value = [];
        for (let key = 0; key < input.length; key++) {
          const value2 = input[key];
          const itemDataset = this.item["~run"]({ value: value2 }, config2);
          if (itemDataset.issues) {
            const pathItem = {
              type: "array",
              origin: "value",
              input,
              key,
              value: value2
            };
            for (const issue of itemDataset.issues) {
              if (issue.path) {
                issue.path.unshift(pathItem);
              } else {
                issue.path = [pathItem];
              }
              dataset.issues?.push(issue);
            }
            if (!dataset.issues) {
              dataset.issues = itemDataset.issues;
            }
            if (config2.abortEarly) {
              dataset.typed = false;
              break;
            }
          }
          if (!itemDataset.typed) {
            dataset.typed = false;
          }
          dataset.value.push(itemDataset.value);
        }
      } else {
        _addIssue(this, "type", dataset, config2);
      }
      return dataset;
    }
  };
}
// @__NO_SIDE_EFFECTS__
function boolean(message) {
  return {
    kind: "schema",
    type: "boolean",
    reference: boolean,
    expects: "boolean",
    async: false,
    message,
    get "~standard"() {
      return /* @__PURE__ */ _getStandardProps(this);
    },
    "~run"(dataset, config2) {
      if (typeof dataset.value === "boolean") {
        dataset.typed = true;
      } else {
        _addIssue(this, "type", dataset, config2);
      }
      return dataset;
    }
  };
}
// @__NO_SIDE_EFFECTS__
function object(entries, message) {
  return {
    kind: "schema",
    type: "object",
    reference: object,
    expects: "Object",
    async: false,
    entries,
    message,
    get "~standard"() {
      return /* @__PURE__ */ _getStandardProps(this);
    },
    "~run"(dataset, config2) {
      const input = dataset.value;
      if (input && typeof input === "object") {
        dataset.typed = true;
        dataset.value = {};
        for (const key in this.entries) {
          const valueSchema = this.entries[key];
          if (key in input || (valueSchema.type === "exact_optional" || valueSchema.type === "optional" || valueSchema.type === "nullish") && // @ts-expect-error
          valueSchema.default !== void 0) {
            const value2 = key in input ? (
              // @ts-expect-error
              input[key]
            ) : /* @__PURE__ */ getDefault(valueSchema);
            const valueDataset = valueSchema["~run"]({ value: value2 }, config2);
            if (valueDataset.issues) {
              const pathItem = {
                type: "object",
                origin: "value",
                input,
                key,
                value: value2
              };
              for (const issue of valueDataset.issues) {
                if (issue.path) {
                  issue.path.unshift(pathItem);
                } else {
                  issue.path = [pathItem];
                }
                dataset.issues?.push(issue);
              }
              if (!dataset.issues) {
                dataset.issues = valueDataset.issues;
              }
              if (config2.abortEarly) {
                dataset.typed = false;
                break;
              }
            }
            if (!valueDataset.typed) {
              dataset.typed = false;
            }
            dataset.value[key] = valueDataset.value;
          } else if (valueSchema.fallback !== void 0) {
            dataset.value[key] = /* @__PURE__ */ getFallback(valueSchema);
          } else if (valueSchema.type !== "exact_optional" && valueSchema.type !== "optional" && valueSchema.type !== "nullish") {
            _addIssue(this, "key", dataset, config2, {
              input: void 0,
              expected: `"${key}"`,
              path: [
                {
                  type: "object",
                  origin: "key",
                  input,
                  key,
                  // @ts-expect-error
                  value: input[key]
                }
              ]
            });
            if (config2.abortEarly) {
              break;
            }
          }
        }
      } else {
        _addIssue(this, "type", dataset, config2);
      }
      return dataset;
    }
  };
}
// @__NO_SIDE_EFFECTS__
function optional(wrapped, default_) {
  return {
    kind: "schema",
    type: "optional",
    reference: optional,
    expects: `(${wrapped.expects} | undefined)`,
    async: false,
    wrapped,
    default: default_,
    get "~standard"() {
      return /* @__PURE__ */ _getStandardProps(this);
    },
    "~run"(dataset, config2) {
      if (dataset.value === void 0) {
        if (this.default !== void 0) {
          dataset.value = /* @__PURE__ */ getDefault(this, dataset, config2);
        }
        if (dataset.value === void 0) {
          dataset.typed = true;
          return dataset;
        }
      }
      return this.wrapped["~run"](dataset, config2);
    }
  };
}
// @__NO_SIDE_EFFECTS__
function string(message) {
  return {
    kind: "schema",
    type: "string",
    reference: string,
    expects: "string",
    async: false,
    message,
    get "~standard"() {
      return /* @__PURE__ */ _getStandardProps(this);
    },
    "~run"(dataset, config2) {
      if (typeof dataset.value === "string") {
        dataset.typed = true;
      } else {
        _addIssue(this, "type", dataset, config2);
      }
      return dataset;
    }
  };
}
function parse(schema, input, config2) {
  const dataset = schema["~run"]({ value: input }, /* @__PURE__ */ getGlobalConfig(config2));
  if (dataset.issues) {
    throw new ValiError(dataset.issues);
  }
  return dataset.value;
}

// ../shared/src/schemas.ts
var CustomComponentPropSchema = object({
  /**
   * The name of the prop in the user-defined component.
   * @example
   * "to"
   */
  name: string(),
  /**
   * The name of the prop in the built-in component.
   * @example
   * "href"
   */
  as: optional(string()),
  /**
   * Whether the prop is controlled or not in the user-defined component.
   * @internal
   * @example
   * `true`
   */
  controlled: optional(boolean()),
  /**
   * The default value of the prop in the user-defined component.
   * @example
   * `"/"`
   */
  defaultValue: optional(string())
});
var CustomComponentSchema = object({
  /**
   * The name of the user-defined component.
   * @example
   * "Link"
   */
  name: string(),
  /**
   * The name of the built-in component that the user-defined component represents.
   * @example
   * "a"
   */
  as: optional(string()),
  /**
   * Attributes mapping between the user-defined component and the built-in component.
   * @example
   * `Link` component has a `to` attribute that represents the `href` attribute in the built-in `a` element with a default value of `"/"`.
   */
  attributes: optional(array(CustomComponentPropSchema)),
  /**
   * The ESQuery selector to select the component precisely.
   * @internal
   * @example
   * `JSXElement:has(JSXAttribute[name.name='component'][value.value='a'])`
   */
  selector: optional(string())
});
var CustomHooksSchema = object({
  use: optional(array(string())),
  useActionState: optional(array(string())),
  useCallback: optional(array(string())),
  useContext: optional(array(string())),
  useDebugValue: optional(array(string())),
  useDeferredValue: optional(array(string())),
  useEffect: optional(array(string())),
  useFormStatus: optional(array(string())),
  useId: optional(array(string())),
  useImperativeHandle: optional(array(string())),
  useInsertionEffect: optional(array(string())),
  useLayoutEffect: optional(array(string())),
  useMemo: optional(array(string())),
  useOptimistic: optional(array(string())),
  useReducer: optional(array(string())),
  useRef: optional(array(string())),
  useState: optional(array(string())),
  useSyncExternalStore: optional(array(string())),
  useTransition: optional(array(string()))
});
var ESLintReactSettingsSchema = object({
  /**
   * The source where React is imported from.
   * @description This allows to specify a custom import location for React when not using the official distribution.
   * @default `"react"`
   * @example `"@pika/react"`
   */
  importSource: optional(string()),
  /**
   * The identifier that's used for JSX Element creation.
   * @default `"createElement"`
   * @deprecated
   */
  jsxPragma: optional(string()),
  /**
   * The identifier that's used for JSX fragment elements.
   * @description This should not be a member expression (i.e. use "Fragment" instead of "React.Fragment").
   * @default `"Fragment"`
   * @deprecated
   */
  jsxPragmaFrag: optional(string()),
  /**
   * The name of the prop that is used for polymorphic components.
   * @description This is used to determine the type of the component.
   * @example `"as"`
   */
  polymorphicPropName: optional(string()),
  /**
   * @internal
   */
  strict: optional(boolean(), false),
  /**
   * Check both the shape and the import to determine if an API is from React.
   * @description This can prevent false positives when using a irrelevant third-party library that has similar APIs to React.
   * @default `true`
   */
  skipImportCheck: optional(boolean(), true),
  /**
   * React version to use, "detect" means auto detect React version from the project's dependencies.
   * If `importSource` is specified, an equivalent version of React should be provided here.
   * @example `"18.3.1"`
   * @default `"detect"`
   */
  version: optional(string()),
  /**
   * A object to define additional hooks that are equivalent to the built-in React Hooks.
   * @description ESLint React will recognize these aliases as equivalent to the built-in hooks in all its rules.
   * @example `{ useEffect: ["useIsomorphicLayoutEffect"] }`
   */
  additionalHooks: optional(CustomHooksSchema),
  /**
   * An array of user-defined components
   * @description This is used to inform the ESLint React plugins how to treat these components during checks.
   * @example `[{ name: "Link", as: "a", attributes: [{ name: "to", as: "href" }, { name: "rel", defaultValue: "noopener noreferrer" }] }]`
   */
  additionalComponents: optional(array(CustomComponentSchema))
});
var DEFAULT_ESLINT_REACT_SETTINGS = {
  ...parse(ESLintReactSettingsSchema, {}),
  importSource: "react"};
function getInstanceId(node, prev) {
  switch (true) {
    case (node.type === AST_NODE_TYPES.VariableDeclarator && node.init === prev):
      return node.id;
    case (node.type === AST_NODE_TYPES.AssignmentExpression && node.right === prev):
      return node.left;
    case (node.type === AST_NODE_TYPES.PropertyDefinition && node.value === prev):
      return node.key;
    case (node.type === AST_NODE_TYPES.BlockStatement || node.type === AST_NODE_TYPES.Program || node.parent === node):
      return _;
    default:
      return getInstanceId(node.parent, node);
  }
}
function getRequireExpressionArguments(node) {
  return match(node).with({ type: AST_NODE_TYPES.CallExpression, arguments: P.select(), callee: { type: AST_NODE_TYPES.Identifier, name: "require" } }, identity).with({ type: AST_NODE_TYPES.MemberExpression, object: P.select() }, getRequireExpressionArguments).otherwise(() => _);
}
function isInitializedFromReact(name, importSource, initialScope) {
  if (name.toLowerCase() === "react") return true;
  const latestDef = VAR.findVariable(name, initialScope)?.defs.at(-1);
  if (latestDef == null) return false;
  const { node, parent } = latestDef;
  if (node.type === AST_NODE_TYPES.VariableDeclarator && node.init != null) {
    const { init } = node;
    if (init.type === AST_NODE_TYPES.MemberExpression && init.object.type === AST_NODE_TYPES.Identifier) {
      return isInitializedFromReact(init.object.name, importSource, initialScope);
    }
    if (init.type === AST_NODE_TYPES.Identifier) {
      return isInitializedFromReact(init.name, importSource, initialScope);
    }
    const args = getRequireExpressionArguments(init);
    const arg0 = args?.[0];
    if (arg0 == null || !AST6.isKindOfLiteral(arg0, "string")) {
      return false;
    }
    return arg0.value === importSource || arg0.value.startsWith(`${importSource}/`);
  }
  return parent?.type === AST_NODE_TYPES.ImportDeclaration && parent.source.value === importSource;
}

// src/utils/is-from-react.ts
var defaultImportSource = DEFAULT_ESLINT_REACT_SETTINGS$1.importSource;
function isFromReactLoose(node, name) {
  switch (node?.type) {
    case AST_NODE_TYPES.Identifier:
      return node.name === name;
    case AST_NODE_TYPES.MemberExpression:
      return node.object.type === AST_NODE_TYPES.Identifier && node.property.type === AST_NODE_TYPES.Identifier && node.property.name === name;
    default:
      return false;
  }
}
function isFromReactStrict(node, name, importSource, initialScope) {
  switch (node?.type) {
    case AST_NODE_TYPES.Identifier:
      return node.name === name && isInitializedFromReact(name, importSource, initialScope);
    case AST_NODE_TYPES.MemberExpression:
      return node.object.type === AST_NODE_TYPES.Identifier && node.property.type === AST_NODE_TYPES.Identifier && node.property.name === name && isInitializedFromReact(node.object.name, importSource, initialScope);
    default:
      return false;
  }
}
function isFromReact(name) {
  return dual(2, (context, node) => {
    if (node == null) return false;
    const { importSource = defaultImportSource, skipImportCheck = true } = unsafeDecodeSettings(context.settings);
    if (skipImportCheck) return isFromReactLoose(node, name);
    return isFromReactStrict(node, name, importSource, context.sourceCode.getScope(node));
  });
}
function isFromReactObject(objectName, propertyName) {
  return dual(2, (context, node) => {
    if (node?.type !== AST_NODE_TYPES.MemberExpression) return false;
    const { importSource = defaultImportSource, skipImportCheck = true } = unsafeDecodeSettings(context.settings);
    const { object: object2, property } = node;
    if (skipImportCheck) return isFromReactLoose(object2, objectName) && isFromReactLoose(property, propertyName);
    return isFromReactStrict(
      object2,
      objectName,
      importSource,
      context.sourceCode.getScope(object2)
    ) && isFromReactLoose(property, propertyName);
  });
}
function isCallFromReact(name) {
  return dual(2, (context, node) => {
    if (node.type !== AST_NODE_TYPES.CallExpression) return false;
    return isFromReact(name)(context, node.callee);
  });
}
function isCallFromReactObject(objectName, propertyName) {
  return dual(2, (context, node) => {
    if (node.type !== AST_NODE_TYPES.CallExpression) return false;
    return isFromReactObject(objectName, propertyName)(context, node.callee);
  });
}
function isInstanceIdEqual(context, a, b) {
  return AST6.isNodeEqual(a, b) || VAR.isNodeValueEqual(a, b, [
    context.sourceCode.getScope(a),
    context.sourceCode.getScope(b)
  ]);
}

// src/utils/is-react-api.ts
function isReactAPI(arg0, arg1) {
  return arg1 == null ? isFromReact(arg0) : isFromReactObject(arg0, arg1);
}
function isReactAPICall(arg0, arg1) {
  return arg1 == null ? isCallFromReact(arg0) : isCallFromReactObject(arg0, arg1);
}
var isChildrenCount = isReactAPI("Children", "count");
var isChildrenForEach = isReactAPI("Children", "forEach");
var isChildrenMap = isReactAPI("Children", "map");
var isChildrenOnly = isReactAPI("Children", "only");
var isChildrenToArray = isReactAPI("Children", "toArray");
var isCloneElement = isReactAPI("cloneElement");
var isCreateContext = isReactAPI("createContext");
var isCreateElement = isReactAPI("createElement");
var isCreateRef = isReactAPI("createRef");
var isForwardRef = isReactAPI("forwardRef");
var isMemo = isReactAPI("memo");
var isChildrenCountCall = isReactAPICall("Children", "count");
var isChildrenForEachCall = isReactAPICall("Children", "forEach");
var isChildrenMapCall = isReactAPICall("Children", "map");
var isChildrenOnlyCall = isReactAPICall("Children", "only");
var isChildrenToArrayCall = isReactAPICall("Children", "toArray");
var isCloneElementCall = isReactAPICall("cloneElement");
var isCreateContextCall = isReactAPICall("createContext");
var isCreateElementCall = isReactAPICall("createElement");
var isCreateRefCall = isReactAPICall("createRef");
var isForwardRefCall = isReactAPICall("forwardRef");
var isMemoCall = isReactAPICall("memo");

// src/hook/is.ts
function isReactHook(node) {
  if (node == null) return false;
  const id = AST6.getFunctionIdentifier(node);
  return id?.name != null && isReactHookName(id.name);
}
function isReactHookCall(node) {
  if (node == null) return false;
  if (node.type !== AST_NODE_TYPES.CallExpression) {
    return false;
  }
  if (node.callee.type === AST_NODE_TYPES.Identifier) {
    return isReactHookName(node.callee.name);
  }
  if (node.callee.type === AST_NODE_TYPES.MemberExpression) {
    return node.callee.property.type === AST_NODE_TYPES.Identifier && isReactHookName(node.callee.property.name);
  }
  return false;
}
function isReactHookCallWithName(context, node) {
  if (node == null || node.type !== AST_NODE_TYPES.CallExpression) return constFalse;
  const {
    importSource = DEFAULT_ESLINT_REACT_SETTINGS.importSource,
    skipImportCheck = true
  } = unsafeDecodeSettings(context.settings);
  const initialScope = context.sourceCode.getScope(node);
  return (name) => {
    switch (true) {
      case (node.callee.type === AST_NODE_TYPES.Identifier && node.callee.name === name):
        return skipImportCheck || isInitializedFromReact(name, importSource, initialScope);
      case (node.callee.type === AST_NODE_TYPES.MemberExpression && node.callee.property.type === AST_NODE_TYPES.Identifier && node.callee.property.name === name && "name" in node.callee.object):
        return skipImportCheck || isInitializedFromReact(node.callee.object.name, importSource, initialScope);
      default:
        return false;
    }
  };
}
function isReactHookCallWithNameLoose(node) {
  if (node == null || node.type !== AST_NODE_TYPES.CallExpression) return constFalse;
  return (name) => {
    switch (node.callee.type) {
      case AST_NODE_TYPES.Identifier:
        return node.callee.name === name;
      case AST_NODE_TYPES.MemberExpression:
        return node.callee.property.type === AST_NODE_TYPES.Identifier && node.callee.property.name === name;
      default:
        return false;
    }
  };
}
function isReactHookCallWithNameAlias(context, name, alias = []) {
  const {
    importSource = DEFAULT_ESLINT_REACT_SETTINGS.importSource,
    skipImportCheck = true
  } = unsafeDecodeSettings(context.settings);
  return (node) => {
    const initialScope = context.sourceCode.getScope(node);
    switch (true) {
      case (node.callee.type === AST_NODE_TYPES.Identifier && node.callee.name === name):
        return skipImportCheck || isInitializedFromReact(name, importSource, initialScope);
      case (node.callee.type === AST_NODE_TYPES.MemberExpression && node.callee.property.type === AST_NODE_TYPES.Identifier && node.callee.property.name === name && "name" in node.callee.object):
        return skipImportCheck || isInitializedFromReact(node.callee.object.name, importSource, initialScope);
      default:
        return alias.some(isReactHookCallWithNameLoose(node));
    }
  };
}
function isUseEffectCallLoose(node) {
  if (node == null) return false;
  if (node.type !== AST_NODE_TYPES.CallExpression) {
    return false;
  }
  switch (node.callee.type) {
    case AST_NODE_TYPES.Identifier:
      return /^use\w*Effect$/u.test(node.callee.name);
    case AST_NODE_TYPES.MemberExpression:
      return node.callee.property.type === AST_NODE_TYPES.Identifier && /^use\w*Effect$/u.test(node.callee.property.name);
    default:
      return false;
  }
}
var isUseCall = flip(isReactHookCallWithName)("use");
var isUseActionStateCall = flip(isReactHookCallWithName)("useActionState");
var isUseCallbackCall = flip(isReactHookCallWithName)("useCallback");
var isUseContextCall = flip(isReactHookCallWithName)("useContext");
var isUseDebugValueCall = flip(isReactHookCallWithName)("useDebugValue");
var isUseDeferredValueCall = flip(isReactHookCallWithName)("useDeferredValue");
var isUseEffectCall = flip(isReactHookCallWithName)("useEffect");
var isUseFormStatusCall = flip(isReactHookCallWithName)("useFormStatus");
var isUseIdCall = flip(isReactHookCallWithName)("useId");
var isUseImperativeHandleCall = flip(isReactHookCallWithName)("useImperativeHandle");
var isUseInsertionEffectCall = flip(isReactHookCallWithName)("useInsertionEffect");
var isUseLayoutEffectCall = flip(isReactHookCallWithName)("useLayoutEffect");
var isUseMemoCall = flip(isReactHookCallWithName)("useMemo");
var isUseOptimisticCall = flip(isReactHookCallWithName)("useOptimistic");
var isUseReducerCall = flip(isReactHookCallWithName)("useReducer");
var isUseRefCall = flip(isReactHookCallWithName)("useRef");
var isUseStateCall = flip(isReactHookCallWithName)("useState");
var isUseSyncExternalStoreCall = flip(isReactHookCallWithName)("useSyncExternalStore");
var isUseTransitionCall = flip(isReactHookCallWithName)("useTransition");

// src/hook/hook-collector.ts
function useHookCollector() {
  const hooks = /* @__PURE__ */ new Map();
  const functionEntries = [];
  const onFunctionEnter = (node) => {
    const id = AST6.getFunctionIdentifier(node);
    const key = getId();
    const name = id?.name;
    if (name != null && isReactHookName(name)) {
      functionEntries.push({ key, node, isHook: true });
      hooks.set(key, {
        id,
        key,
        kind: "function",
        name,
        node,
        flag: 0n,
        hint: 0n,
        hookCalls: []
      });
      return;
    }
    functionEntries.push({ key, node, isHook: false });
  };
  const onFunctionExit = () => {
    functionEntries.pop();
  };
  const ctx = {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getAllHooks(node) {
      return hooks;
    }
  };
  const listeners = {
    ":function[type]": onFunctionEnter,
    ":function[type]:exit": onFunctionExit,
    "CallExpression[type]"(node) {
      if (!isReactHookCall(node)) {
        return;
      }
      const fEntry = functionEntries.at(-1);
      if (fEntry?.key == null) {
        return;
      }
      const hook = hooks.get(fEntry.key);
      if (hook == null) {
        return;
      }
      hook.hookCalls.push(node);
    }
  };
  return { ctx, listeners };
}
var ERComponentHint = {
  /**
   * 1n << 0n - 1n << 63n are reserved for JSXValueHint
   */
  ...JSX3.JSXValueHint,
  /**
   * Skip function component created by React.memo
   */
  SkipMemo: 1n << 64n,
  /**
   * Skip function component created by React.forwardRef
   */
  SkipForwardRef: 1n << 65n,
  /**
   * Skip function component defined as array map argument
   */
  SkipArrayMapArgument: 1n << 66n,
  /**
   * Skip function component defined on object method
   */
  SkipObjectMethod: 1n << 67n,
  /**
   * Skip function component defined on class method
   */
  SkipClassMethod: 1n << 68n,
  /**
   * Skip function component defined on class property
   */
  SkipClassProperty: 1n << 69n
};
var DEFAULT_COMPONENT_HINT = 0n | ERComponentHint.SkipBooleanLiteral | ERComponentHint.SkipEmptyArray | ERComponentHint.SkipArrayMapArgument | ERComponentHint.SkipNumberLiteral | ERComponentHint.SkipStringLiteral | ERComponentHint.SkipUndefined | ERComponentHint.StrictArray | ERComponentHint.StrictConditional | ERComponentHint.StrictLogical;

// src/component/component-flag.ts
var ERComponentFlag = {
  None: 0n,
  PureComponent: 1n << 0n,
  CreateElement: 1n << 1n,
  Memo: 1n << 2n,
  ForwardRef: 1n << 3n,
  Async: 1n << 4n
};
function isComponentWrapperCall(context, node) {
  if (node.type !== AST_NODE_TYPES.CallExpression) {
    return false;
  }
  return isMemoCall(context, node) || isForwardRefCall(context, node) || isReactHookCallWithNameLoose(node)("useCallback");
}
function getFunctionComponentId(context, node) {
  const functionId = AST6.getFunctionIdentifier(node);
  if (functionId != null) {
    return functionId;
  }
  const { parent } = node;
  if (parent.type === AST_NODE_TYPES.CallExpression && isComponentWrapperCall(context, parent) && parent.parent.type === AST_NODE_TYPES.VariableDeclarator && parent.parent.id.type === AST_NODE_TYPES.Identifier) {
    return parent.parent.id;
  }
  if (parent.type === AST_NODE_TYPES.CallExpression && isComponentWrapperCall(context, parent) && parent.parent.type === AST_NODE_TYPES.CallExpression && isComponentWrapperCall(context, parent.parent) && parent.parent.parent.type === AST_NODE_TYPES.VariableDeclarator && parent.parent.parent.id.type === AST_NODE_TYPES.Identifier) {
    return parent.parent.parent.id;
  }
  return _;
}
var RE_COMPONENT_NAME = /^[A-Z]/u;
var RE_COMPONENT_NAME_LOOSE = /^_?[A-Z]/u;
function isComponentName(name) {
  return RE_COMPONENT_NAME.test(name);
}
function isComponentNameLoose(name) {
  return RE_COMPONENT_NAME_LOOSE.test(name);
}
function getComponentNameFromId(id) {
  if (id == null) return _;
  return Array.isArray(id) ? id.map((n) => n.name).join(".") : id.name;
}
function hasNoneOrLooseComponentName(context, fn) {
  const id = getFunctionComponentId(context, fn);
  if (id == null) return true;
  const name = Array.isArray(id) ? id.at(-1)?.name : id.name;
  return name != null && isComponentNameLoose(name);
}
function isClassComponent(node) {
  if ("superClass" in node && node.superClass != null) {
    const re = /^(?:Pure)?Component$/u;
    switch (true) {
      case node.superClass.type === AST_NODE_TYPES.Identifier:
        return re.test(node.superClass.name);
      case (node.superClass.type === AST_NODE_TYPES.MemberExpression && node.superClass.property.type === AST_NODE_TYPES.Identifier):
        return re.test(node.superClass.property.name);
    }
  }
  return false;
}
function isPureComponent(node) {
  if ("superClass" in node && node.superClass != null) {
    const re = /^PureComponent$/u;
    switch (true) {
      case node.superClass.type === AST_NODE_TYPES.Identifier:
        return re.test(node.superClass.name);
      case (node.superClass.type === AST_NODE_TYPES.MemberExpression && node.superClass.property.type === AST_NODE_TYPES.Identifier):
        return re.test(node.superClass.property.name);
    }
  }
  return false;
}

// src/component/component-lifecycle.ts
function isComponentDidMount(node) {
  return AST6.isMethodOrProperty(node) && node.key.type === AST_NODE_TYPES.Identifier && node.key.name === "componentDidMount";
}
function isComponentWillUnmount(node) {
  return AST6.isMethodOrProperty(node) && node.key.type === AST_NODE_TYPES.Identifier && node.key.name === "componentWillUnmount";
}
function isComponentDidCatch(node) {
  return AST6.isMethodOrProperty(node) && !node.static && node.key.type === AST_NODE_TYPES.Identifier && node.key.name === "componentDidCatch";
}
function isGetDerivedStateFromError(node) {
  return AST6.isMethodOrProperty(node) && node.static && node.key.type === AST_NODE_TYPES.Identifier && node.key.name === "getDerivedStateFromError";
}
function isGetDerivedStateFromProps(node) {
  return AST6.isMethodOrProperty(node) && node.static && node.key.type === AST_NODE_TYPES.Identifier && node.key.name === "getDerivedStateFromProps";
}
function isFunctionOfComponentDidMount(node) {
  return AST6.isFunction(node) && isComponentDidMount(node.parent) && node.parent.value === node;
}
function isFunctionOfComponentWillUnmount(node) {
  return AST6.isFunction(node) && isComponentWillUnmount(node.parent) && node.parent.value === node;
}
function isRenderMethodLike(node) {
  return AST6.isMethodOrProperty(node) && node.key.type === AST_NODE_TYPES.Identifier && node.key.name === "render" && node.parent.parent.type === AST_NODE_TYPES.ClassDeclaration;
}
function isFunctionOfRenderMethod(node) {
  if (!isRenderMethodLike(node.parent)) {
    return false;
  }
  return isClassComponent(node.parent.parent.parent);
}
function isInsideRenderMethod(node) {
  return AST6.findParentNode(node, (node2) => isRenderMethodLike(node2) && isClassComponent(node2.parent.parent)) != null;
}

// src/component/hierarchy.ts
function hasValidHierarchy(context, node, hint) {
  if (isChildrenOfCreateElement(context, node) || isFunctionOfRenderMethod(node)) {
    return false;
  }
  if (hint & ERComponentHint.SkipObjectMethod && isFunctionOfObjectMethod(node.parent)) {
    return false;
  }
  if (hint & ERComponentHint.SkipClassMethod && isFunctionOfClassMethod(node.parent)) {
    return false;
  }
  if (hint & ERComponentHint.SkipClassProperty && isFunctionOfClassProperty(node.parent)) {
    return false;
  }
  if (hint & ERComponentHint.SkipArrayMapArgument && AST6.isArrayMapCallLoose(node.parent)) {
    return false;
  }
  const boundaryNode = AST6.findParentNode(
    node,
    AST6.isOneOf([
      AST_NODE_TYPES.JSXExpressionContainer,
      AST_NODE_TYPES.ArrowFunctionExpression,
      AST_NODE_TYPES.FunctionExpression,
      AST_NODE_TYPES.Property,
      AST_NODE_TYPES.ClassBody
    ])
  );
  return boundaryNode == null || boundaryNode.type !== AST_NODE_TYPES.JSXExpressionContainer;
}
function isChildrenOfCreateElement(context, node) {
  const parent = node.parent;
  if (parent == null || parent.type !== AST_NODE_TYPES.CallExpression) return false;
  if (!isCreateElementCall(context, parent)) return false;
  return parent.arguments.slice(2).some((arg) => arg === node);
}
function isFunctionOfClassMethod(node) {
  return (node.type === AST_NODE_TYPES.FunctionExpression || node.type === AST_NODE_TYPES.ArrowFunctionExpression) && node.parent.type === AST_NODE_TYPES.MethodDefinition;
}
function isFunctionOfClassProperty(node) {
  return (node.type === AST_NODE_TYPES.FunctionExpression || node.type === AST_NODE_TYPES.ArrowFunctionExpression) && node.parent.type === AST_NODE_TYPES.Property;
}
function isFunctionOfObjectMethod(node) {
  return (node.type === AST_NODE_TYPES.FunctionExpression || node.type === AST_NODE_TYPES.ArrowFunctionExpression) && node.parent.type === AST_NODE_TYPES.Property && node.parent.parent.type === AST_NODE_TYPES.ObjectExpression;
}

// src/component/component-collector.ts
function useComponentCollector(context, options = {}) {
  const {
    collectDisplayName = false,
    collectHookCalls = false,
    hint = DEFAULT_COMPONENT_HINT
  } = options;
  const jsxCtx = { getScope: (node) => context.sourceCode.getScope(node) };
  const components = /* @__PURE__ */ new Map();
  const functionEntries = [];
  const getCurrentEntry = () => functionEntries.at(-1);
  const onFunctionEnter = (node) => {
    const key = getId();
    functionEntries.push({ key, node, hookCalls: [], isComponent: false });
  };
  const onFunctionExit = () => {
    const entry = functionEntries.at(-1);
    if (entry == null) return;
    if (!entry.isComponent) return functionEntries.pop();
    const shouldDrop = AST6.getNestedReturnStatements(entry.node.body).slice().reverse().some((r) => {
      return context.sourceCode.getScope(r).block === entry.node && r.argument != null && !JSX3.isJSXValue(r.argument, jsxCtx, hint);
    });
    if (shouldDrop) {
      components.delete(entry.key);
    }
    return functionEntries.pop();
  };
  const ctx = {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getAllComponents(node) {
      return components;
    },
    getCurrentEntries() {
      return [...functionEntries];
    },
    getCurrentEntry
  };
  const listeners = {
    ":function[type]": onFunctionEnter,
    ":function[type]:exit": onFunctionExit,
    "ArrowFunctionExpression[type][body.type!='BlockStatement']"() {
      const entry = getCurrentEntry();
      if (entry == null) return;
      const { body } = entry.node;
      const isComponent = hasNoneOrLooseComponentName(context, entry.node) && JSX3.isJSXValue(body, jsxCtx, hint) && hasValidHierarchy(context, entry.node, hint);
      if (!isComponent) return;
      const initPath = AST6.getFunctionInitPath(entry.node);
      const id = getFunctionComponentId(context, entry.node);
      const name = getComponentNameFromId(id);
      const key = getId();
      components.set(key, {
        id,
        key,
        kind: "function",
        name,
        node: entry.node,
        displayName: _,
        flag: getComponentFlag(initPath),
        hint,
        hookCalls: entry.hookCalls,
        initPath
      });
    },
    ...collectDisplayName ? {
      [DISPLAY_NAME_ASSIGNMENT_SELECTOR](node) {
        const { left, right } = node;
        if (left.type !== AST_NODE_TYPES.MemberExpression) return;
        const componentName = left.object.type === AST_NODE_TYPES.Identifier ? left.object.name : _;
        const component = [...components.values()].findLast(({ name }) => name != null && name === componentName);
        if (component == null) return;
        component.displayName = right;
      }
    } : {},
    ...collectHookCalls ? {
      "CallExpression[type]:exit"(node) {
        if (!isReactHookCall(node)) return;
        const entry = getCurrentEntry();
        if (entry == null) return;
        entry.hookCalls.push(node);
      }
    } : {},
    "ReturnStatement[type]"(node) {
      const entry = getCurrentEntry();
      if (entry == null) return;
      const isComponent = hasNoneOrLooseComponentName(context, entry.node) && JSX3.isJSXValue(node.argument, jsxCtx, hint) && hasValidHierarchy(context, entry.node, hint);
      if (!isComponent) return;
      entry.isComponent = true;
      const initPath = AST6.getFunctionInitPath(entry.node);
      const id = getFunctionComponentId(context, entry.node);
      const name = getComponentNameFromId(id);
      components.set(entry.key, {
        id,
        key: entry.key,
        kind: "function",
        name,
        node: entry.node,
        displayName: _,
        flag: getComponentFlag(initPath),
        hint,
        hookCalls: entry.hookCalls,
        initPath
      });
    }
  };
  return { ctx, listeners };
}
function getComponentFlag(initPath) {
  let flag = ERComponentFlag.None;
  if (initPath != null && AST6.hasCallInFunctionInitPath("memo", initPath)) {
    flag |= ERComponentFlag.Memo;
  }
  if (initPath != null && AST6.hasCallInFunctionInitPath("forwardRef", initPath)) {
    flag |= ERComponentFlag.ForwardRef;
  }
  return flag;
}
function useComponentCollectorLegacy() {
  const components = /* @__PURE__ */ new Map();
  const ctx = {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getAllComponents(node) {
      return components;
    }
  };
  const collect = (node) => {
    if (!isClassComponent(node)) {
      return;
    }
    const id = AST6.getClassIdentifier(node);
    const key = getId();
    const flag = isPureComponent(node) ? ERComponentFlag.PureComponent : ERComponentFlag.None;
    components.set(
      key,
      {
        id,
        key,
        kind: "class",
        name: id?.name,
        node,
        // TODO: Get displayName of class component
        displayName: _,
        flag,
        hint: 0n,
        // TODO: Get methods of class component
        methods: []
      }
    );
  };
  const listeners = {
    "ClassDeclaration[type]": collect,
    "ClassExpression[type]": collect
  };
  return { ctx, listeners };
}
var ERPhaseRelevance = birecord({
  mount: "unmount",
  setup: "cleanup"
});
var isInversePhase = dual(2, (a, b) => ERPhaseRelevance.get(a) === b);
function isRenderFunctionLoose(context, node) {
  const { body, parent } = node;
  if (AST6.getFunctionIdentifier(node)?.name.startsWith("render")) {
    return parent.type === AST_NODE_TYPES.JSXExpressionContainer && parent.parent.type === AST_NODE_TYPES.JSXAttribute && parent.parent.name.type === AST_NODE_TYPES.JSXIdentifier && parent.parent.name.name.startsWith("render");
  }
  return JSX3.isJSXValue(
    body,
    {
      getScope: (node2) => context.sourceCode.getScope(node2)
    },
    JSX3.JSXValueHint.SkipNullLiteral | JSX3.JSXValueHint.SkipUndefined | JSX3.JSXValueHint.StrictLogical | JSX3.JSXValueHint.StrictConditional
  );
}
function isRenderPropLoose(context, node) {
  if (node.name.type !== AST_NODE_TYPES.JSXIdentifier) {
    return false;
  }
  return node.name.name.startsWith("render") && node.value?.type === AST_NODE_TYPES.JSXExpressionContainer && AST6.isFunction(node.value.expression) && isRenderFunctionLoose(context, node.value.expression);
}
function isDirectValueOfRenderPropertyLoose(node) {
  const matching = (node2) => {
    return node2.type === AST_NODE_TYPES.Property && node2.key.type === AST_NODE_TYPES.Identifier && node2.key.name.startsWith("render");
  };
  return matching(node) || node.parent != null && matching(node.parent);
}
function isDeclaredInRenderPropLoose(node) {
  if (isDirectValueOfRenderPropertyLoose(node)) {
    return true;
  }
  const parent = AST6.findParentNode(node, AST6.is(AST_NODE_TYPES.JSXExpressionContainer))?.parent;
  if (parent?.type !== AST_NODE_TYPES.JSXAttribute) {
    return false;
  }
  return parent.name.type === AST_NODE_TYPES.JSXIdentifier && parent.name.name.startsWith("render");
}
function isThisSetState(node) {
  const { callee } = node;
  return callee.type === AST_NODE_TYPES.MemberExpression && AST6.isThisExpression(callee.object) && callee.property.type === AST_NODE_TYPES.Identifier && callee.property.name === "setState";
}
function isAssignmentToThisState(node) {
  const { left } = node;
  return left.type === AST_NODE_TYPES.MemberExpression && AST6.isThisExpression(left.object) && AST6.getPropertyName(left.property) === "state";
}
function isFunctionOfUseEffectSetup(node) {
  if (node == null) return false;
  return node.parent?.type === AST_NODE_TYPES.CallExpression && node.parent.callee !== node && node.parent.callee.type === AST_NODE_TYPES.Identifier && node.parent.arguments.at(0) === node && isUseEffectCallLoose(node.parent);
}
function isFunctionOfUseEffectCleanup(node) {
  if (node == null) return false;
  const returnStatement = AST6.findParentNode(node, AST6.is(AST_NODE_TYPES.ReturnStatement));
  const enclosingFunction = AST6.findParentNode(node, AST6.isFunction);
  const functionOfReturnStatement = AST6.findParentNode(returnStatement, AST6.isFunction);
  return enclosingFunction === functionOfReturnStatement && isFunctionOfUseEffectSetup(enclosingFunction);
}

export { DEFAULT_COMPONENT_HINT, DISPLAY_NAME_ASSIGNMENT_SELECTOR, ERComponentFlag, ERComponentHint, ERPhaseRelevance, RE_COMPONENT_NAME, RE_COMPONENT_NAME_LOOSE, RE_HOOK_NAME, getComponentNameFromId, getFunctionComponentId, getInstanceId, hasNoneOrLooseComponentName, hasValidHierarchy, isAssignmentToThisState, isCallFromReact, isCallFromReactObject, isChildrenCount, isChildrenCountCall, isChildrenForEach, isChildrenForEachCall, isChildrenMap, isChildrenMapCall, isChildrenOnly, isChildrenOnlyCall, isChildrenToArray, isChildrenToArrayCall, isClassComponent, isCloneElement, isCloneElementCall, isComponentDidCatch, isComponentDidMount, isComponentName, isComponentNameLoose, isComponentWillUnmount, isCreateContext, isCreateContextCall, isCreateElement, isCreateElementCall, isCreateRef, isCreateRefCall, isDeclaredInRenderPropLoose, isDirectValueOfRenderPropertyLoose, isForwardRef, isForwardRefCall, isFromReact, isFromReactLoose, isFromReactObject, isFromReactStrict, isFunctionOfComponentDidMount, isFunctionOfComponentWillUnmount, isFunctionOfRenderMethod, isFunctionOfUseEffectCleanup, isFunctionOfUseEffectSetup, isGetDerivedStateFromError, isGetDerivedStateFromProps, isInitializedFromReact, isInsideRenderMethod, isInstanceIdEqual, isInversePhase, isMemo, isMemoCall, isPureComponent, isReactAPI, isReactAPICall, isReactHook, isReactHookCall, isReactHookCallWithName, isReactHookCallWithNameAlias, isReactHookCallWithNameLoose, isReactHookName, isReactHookNameLoose, isRenderFunctionLoose, isRenderMethodLike, isRenderPropLoose, isThisSetState, isUseActionStateCall, isUseCall, isUseCallbackCall, isUseContextCall, isUseDebugValueCall, isUseDeferredValueCall, isUseEffectCall, isUseEffectCallLoose, isUseFormStatusCall, isUseIdCall, isUseImperativeHandleCall, isUseInsertionEffectCall, isUseLayoutEffectCall, isUseMemoCall, isUseOptimisticCall, isUseReducerCall, isUseRefCall, isUseStateCall, isUseSyncExternalStoreCall, isUseTransitionCall, useComponentCollector, useComponentCollectorLegacy, useHookCollector };
